@@ -169,7 +169,7 @@ short decoder( short linewidth)
 #ifndef XFRACT
 	static short far sizeofstring[MAX_CODES+1];	/* size of string list */
 #else
-	extern int prefix[];
+	unsigned short prefix[MAX_CODES+1];	/* Prefix linked list */
 	short sizeofstring[MAX_CODES+1];	/* size of string list */
 #endif
 	BYTE *sp;
