@@ -5,6 +5,7 @@ extern long lmagnitud, llimit, llimit2, lclosenuff, l16triglim;
 extern int overflow, bitshift;
 extern _LCMPLX lold, lnew;
 
+#if 0
 longbailout()
 {
     ltempsqrx = lsqr(lnew.x); ltempsqry = lsqr(lnew.y);
@@ -16,6 +17,7 @@ longbailout()
     lold = lnew;
     return(0);
 }
+#endif
 
 FManOWarfpFractal() {}
 FJuliafpFractal() {}
