@@ -7,9 +7,11 @@
 
 #include <string.h>
 #ifndef NOBSTRING
+#ifndef sun
 /* If this gives you an error, read the README and modify the Makefile. */
 #include <bstring.h>
 #endif
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -613,7 +615,7 @@ decode_fractal_info(info,dir)
 	strncpy((char *)bufPtr,info->info_id,8);
     }
     bufPtr += 8;
-    getInt(&info->iterations,&bufPtr,dir);
+    getInt(&info->iterationsold,&bufPtr,dir);
     getInt(&info->fractal_type,&bufPtr,dir);
     getDouble(&info->xmin,&bufPtr,dir);
     getDouble(&info->xmax,&bufPtr,dir);
@@ -670,7 +672,7 @@ decode_fractal_info(info,dir)
     getLong(&info->tot_extend_len,&bufPtr,dir);
     getInt(&info->distest,&bufPtr,dir);
     getInt(&info->floatflag,&bufPtr,dir);
-    getInt(&info->bailout,&bufPtr,dir);
+    getInt(&info->bailoutold,&bufPtr,dir);
     getLong(&info->calctime,&bufPtr,dir);
     for (i=0;i<4;i++) {
 	getChar(&info->trigndx[i],&bufPtr,dir);
@@ -716,14 +718,21 @@ decode_fractal_info(info,dir)
     getDouble(&info->dparm8,&bufPtr,dir);
     getDouble(&info->dparm9,&bufPtr,dir);
     getDouble(&info->dparm10,&bufPtr,dir);
-    for (i=0;i<50;i++) {
-	getInt(&info->future[i],&bufPtr,dir);
-    }
+    getLong(&info->bailout,&bufPtr,dir);  
+    getInt(&info->bailoutest,&bufPtr,dir);
+    getLong(&info->iterations,&bufPtr,dir);
+    getInt(&info->bf_math,&bufPtr,dir);
+    getInt(&info->bflength,&bufPtr,dir);
+    getInt(&info->yadjust,&bufPtr,dir); 
+    getInt(&info->old_demm_colors,&bufPtr,dir);
+    for (i=0;i<(sizeof(info->future)/sizeof(short));i++) {
+        getInt(&info->future[i],&bufPtr,dir);
+    }   
     if (bufPtr-buf != FRACTAL_INFO_SIZE) {
 	printf("Warning: loadfile miscount on fractal_info structure.\n");
 	printf("Components add up to %d bytes, but FRACTAL_INFO_SIZE = %d\n",
 		bufPtr-buf, FRACTAL_INFO_SIZE);
-    }
+    } 
     if (dir==0) {
 	bcopy((char *)buf,(char *)info,FRACTAL_INFO_SIZE);
     }
