@@ -39,6 +39,10 @@
 #ifdef linux
 #define FNDELAY O_NDELAY
 #endif
+#ifdef __SVR4
+# include <sys/filio.h>
+# define FNDELAY O_NONBLOCK
+#endif
 
 /* Check if there is a character waiting for us.  */
 #define input_pending() (ioctl(0,FIONREAD,&iocount),(int)iocount)
@@ -234,7 +238,11 @@ UnixInit()
 	text_type = 1;
     }
 
-    signal(SIGINT,goodbye);
+    initdacbox();
+
+    if (!simple_input) {
+	  signal(SIGINT,(sigfunc *)goodbye);
+    }
     signal(SIGFPE, fpe_handler);
     /*
     signal(SIGTSTP,goodbye);
@@ -518,7 +526,7 @@ initUnixWindow()
 	xsync();
     }
 
-    initdacbox();
+    writevideopalette();
 
     videotable[0].xdots = sxdots;
     videotable[0].ydots = sydots;
@@ -639,20 +647,14 @@ static void
 initdacbox()
 {
     int i;
-    if (colors==2) {
-	dacbox[0][0] = dacbox[0][1] = dacbox[0][2] = 0;
-	dacbox[1][0] = dacbox[1][1] = dacbox[1][2] = 63;
-    } else {
-	for (i=0;i<256;i++) {
-            dacbox[i][0] = (i>>5)*8+7;
-            dacbox[i][1] = (((i+16)&28)>>2)*8+7;
-            dacbox[i][2] = (((i+2)&3))*16+15;
-	}
-	dacbox[0][0] = dacbox[0][1] = dacbox[0][2] = 0;
-	dacbox[1][0] = dacbox[1][1] = dacbox[1][2] = 63;
-	dacbox[2][0] = 47; dacbox[2][1] = dacbox[2][2] = 63;
+    for (i=0;i<256;i++) {
+	dacbox[i][0] = (i>>5)*8+7;
+	dacbox[i][1] = (((i+16)&28)>>2)*8+7;
+	dacbox[i][2] = (((i+2)&3))*16+15;
     }
-    writevideopalette();
+    dacbox[0][0] = dacbox[0][1] = dacbox[0][2] = 0;
+    dacbox[1][0] = dacbox[1][1] = dacbox[1][2] = 63;
+    dacbox[2][0] = 47; dacbox[2][1] = dacbox[2][2] = 63;
 }
 
 
@@ -918,9 +920,10 @@ BYTE *pixels;
  *
  *----------------------------------------------------------------------
  */
-void writevideo(int x, int y, int color)
+void writevideo(x,y,color)
+int x,y,color;
 {
-#if DEBUG /* Debugging checks */
+#ifdef DEBUG /* Debugging checks */
     if (color>=colors || color < 0) {
 	printf("Color %d too big %d\n", color, colors);
     }
@@ -1196,18 +1199,10 @@ int block;
 	FD_ZERO(&reads);
 	FD_SET(0,&reads);
 	if (unixDisk) {
-#ifdef __hpux
-	    status = select(1,(int *)&reads,NULL,NULL,&tout);
-#else
 	    status = select(1,&reads,NULL,NULL,&tout);
-#endif
 	} else {
 	    FD_SET(ConnectionNumber(Xdp),&reads);
-#ifdef __hpux
-	    status = select(ConnectionNumber(Xdp)+1,(int *)&reads,NULL,NULL,&tout);
-#else
 	    status = select(ConnectionNumber(Xdp)+1,&reads,NULL,NULL,&tout);
-#endif
 	}
 	if (status<=0) {
 	    return 0;
@@ -1513,10 +1508,6 @@ handleesc()
 #endif
 }
 
-
-extern double zwidth, zdepth, zrotate, zskew, zbx, zby, dxsize,dysize;
-extern int zoomoff;
-
 extern int editpal_cursor;
 extern void Cursor_SetPos();
 
@@ -2077,20 +2068,12 @@ xgetfont()
 void
 shell_to_dos()
 {
-#ifdef __hpux
-    void (*sigint)();
-#else
-    int (*sigint)();
-#endif
+    sigfunc *sigint;
     char *shell;
     char *argv[2];
     int pid, donepid;
 
-#ifdef __hpux
-    sigint = (void(*)())signal(SIGINT, SIG_IGN);
-#else
-    sigint = (int(*)())signal(SIGINT, SIG_IGN);
-#endif
+    sigint = (sigfunc *)signal(SIGINT, SIG_IGN);
     shell = getenv("SHELL");
     if (shell==NULL) {
 	shell = SHELL;
@@ -2138,7 +2121,7 @@ shell_to_dos()
 	fcntl(0,F_SETFL,FNDELAY);
     }
 
-    (void) signal(SIGINT, sigint);
+    signal(SIGINT, (sigfunc *)sigint);
     putchar('\n');
 }
 
@@ -2163,7 +2146,7 @@ schedulealarm(soon)
 int soon;
 {
     if (!fastmode) return;
-    signal(SIGALRM, setredrawscreen);
+    signal(SIGALRM, (sigfunc *)setredrawscreen);
     if (soon) {
 	alarm(1);
     } else {
